/*************
 *
 * Filename:    SLQSsampleCM.c
 *
 * Purpose:     Sample Connection Manager Test Application
 *
 * Copyright: © 2011 Sierra Wireless Inc., all rights reserved
 *
 **************/
#include "SWIWWANCMAPI.h"
#include "qmerrno.h"
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <syslog.h>

#define DEV_NODE_SZ 256
#define DEV_KEY_SZ  16
#define ARGS_MAX    256

/* enums */
enum {
  eSLQSStart,
  eQCWWAN2kEnumerateDevices,
  eQCWWANConnect,
  eQCWWANDisconnect,
  eQCWWANGetConnectedDeviceID,
  eGetSessionState,
  eStartDataSession,
  eStopDataSession,
  ePerformNetworkScan,
  eInitiateNetworkRegistration,
#if 0
  eSetDefaultProfile,
#endif
  eSLQSKillSDKProcess,
  eExit
};

/* user types */
typedef struct device_info_param{
  CHAR deviceNode[256];
  CHAR deviceKey[16];
}device_info_t;

typedef struct session_state_param{
  ULONG sessionid;
  ULONG failurereason;
}session_state_t;

typedef struct networkscan_param{
  WORD  mcc;
  WORD  mnc;
  ULONG inuse;
  ULONG roaming;
  ULONG forbidden;
  ULONG preferred;
  CHAR  description[255];
}networkscan_t;

typedef struct networkreg_param{
  ULONG regType;
  WORD  mcc;
  WORD  mnc;
  ULONG rat;
}networkreg_t;

typedef ULONG (*apiproc)(void*,...);
struct apiProcTable_entry{
  char str[80];
  ULONG (*proc)(void*,...);
};

typedef struct profile_info{
  ULONG  profileType;
  ULONG *pPDPType;
  ULONG *pIPAddress;
  ULONG *pPrimaryDNS;
  ULONG *pSecondaryDNS;
  ULONG *pAuthentication;
  CHAR  *pName;
  CHAR  *pAPNName;
  CHAR  *pUsername;
  CHAR  *pPassword;
}profile_info_t;

typedef struct{
    enum eQCWWANError e;
    const char *es;
}slqserr_s;

/* Prototypes */
ULONG wSLQSStart(void *);
ULONG wQCWWANEnumerateDevices(device_info_t* pdevices);
ULONG wQCWWANConnect(device_info_t* pdev);
ULONG wQCWWANGetConnectedDevice(device_info_t* pdev);
ULONG wGetSessionState(void *);
ULONG wStartDataSession(session_state_t *ss);
ULONG wStopDataSession(ULONG sessionId);
ULONG wPerformNetworkScan(BYTE* pinstanceSize, networkscan_t* pinstances);
ULONG wInitiateNetworkRegistration(networkreg_t *pin);
ULONG wQCWWANDisconnect(void *);
#if 0
ULONG wSetDefaultProfile(profile_info_t *pinf);
#endif
ULONG wSLQSKillSDKProcess(void *);
static int executeApiProc(int);
static const char *slqserrstr(ULONG);

/* macros*/
#define rcprint(s, u) syslog(LOG_USER, "%s: rc = 0x%lX, %s", s, u, slqserrstr(u))

/* path to sdk binary */
static char *sdkbinpath = NULL;

/* critical section vars */
static pthread_cond_t cond;
static pthread_mutex_t mutex;

/* state vars */
static BOOL devready = 0;
static BOOL sdkbound = 0;
static ULONG rc;

/* device connectivity */
static device_info_t devices[1] = { { {'\0'}, {'\0'} } };
static device_info_t *pdev = &devices[0];

/* wireless data service */
static session_state_t ss;

/* network access service */
static BYTE instanceSize = 10;
static networkscan_t instances[10];
static networkreg_t nwreg;

/* wireless data service */
static profile_info_t dfltProf;

/* error strings */
static slqserr_s errstr[] =
{
    { eQCWWAN_ERR_INTERNAL,
      "eQCWWAN_ERR_INTERNAL" },

    { eQCWWAN_ERR_MEMORY,
      "eQCWWAN_ERR_MEMORY" },

    { eQCWWAN_ERR_INVALID_ARG,
      "eQCWWAN_ERR_INVALID_ARG" },

    { eQCWWAN_ERR_BUFFER_SZ,
      "eQCWWAN_ERR_BUFFER_SZ" },

    { eQCWWAN_ERR_NO_DEVICE,
      "eQCWWAN_ERR_NO_DEVICE" },

    { eQCWWAN_ERR_SWIDCS_IOCTL_ERR,
      "eQCWWAN_ERR_SWIDCS_IOCTL_ERR" },

    { eQCWWAN_ERR_SWIDCS_DEVNODE_NOT_FOUND,
      "eQCWWAN_ERR_SWIDCS_DEVNODE_NOT_FOUND" },

    { eQCWWAN_ERR_SWIDCS_IOCTL_ERR,
      "eQCWWAN_ERR_SWIDCS_IOCTL_ERR" },

    { eQCWWAN_ERR_SWIDCS_APP_DISCONNECTED,
      "eQCWWAN_ERR_SWIDCS_APP_DISCONNECTED" },

    { eQCWWAN_ERR_SWICM_QMI_SVC_NOT_SUPPORTED,
      "eQCWWAN_ERR_SWICM_QMI_SVC_NOT_SUPPORTED" },

    { eQCWWAN_ERR_QMI_MISSING_ARG,
      "eQCWWAN_ERR_QMI_MISSING_ARG" },

    { eQCWWAN_ERR_SWICM_SOCKET_IN_USE,
     "eQCWWAN_ERR_SWICM_SOCKET_IN_USE" },

    { 0, "" }
};

/* API table */
static struct apiProcTable_entry apiProcTable[] =
{
  { "SLQSStart",
    (apiproc)wSLQSStart},

  { "QCWWANEnumerateDevices",
    (apiproc)wQCWWANEnumerateDevices},

  { "QCWWANConnect",
    (apiproc)wQCWWANConnect},

  { "QCWWANDisconnect",
    (apiproc)wQCWWANDisconnect},

  { "QCWWANGetConnectedDevice",
    (apiproc)wQCWWANGetConnectedDevice},

  { "GetSessionState",
    (apiproc)wGetSessionState},

  { "StartDataSession",
    (apiproc)wStartDataSession},

  { "StopDataSession",
    (apiproc)wStopDataSession},

  { "PerformNetworkScan",
    (apiproc)wPerformNetworkScan},

  { "InitiateNetworkRegistration",
    (apiproc)wInitiateNetworkRegistration},
#if 0
  { "SetDefaultProfile",
    (apiproc)wSetDefaultProfile},
#endif
  { "SLQSKillSDKProcess",
    (apiproc)wSLQSKillSDKProcess},

  { "Exit",
    NULL },

  { "",
    NULL }
};

/* Functions */
static const char *slqserrstr(ULONG er)
{
    int count = 0;

    while( errstr[count].e ){
        if( errstr[count].e == (enum eQCWWANError)er )
        {
            return errstr[count].es;
        }
        count++;
    }

    return "";
}

static void appstatechange()
{
    /* device is ready and application is already registered with SDK */
    if( devready && sdkbound )
    {
        syslog(LOG_USER, "%s: device ready, APP connected to SDK\n", __func__);
    }
    /* device is ready and application has not registered with SDK */
    else if( devready && !sdkbound )
    {
        syslog(LOG_USER, "%s: device ready, APP disconnected from SDK\n", __func__);
    }
    /* device is disconnected and application has registered with SDK */
    else if( sdkbound )
    {
        syslog(LOG_USER, "%s: device disconnected, APP connected to SDK\n", __func__);
    }
    /* device is disconnected and application has not registered with SDK */
    else
    {
        syslog(LOG_USER, "%s: device disconnected, APP disconnected from SDK\n", __func__);
    }
}

static void devstatecb(eDevState device_state)
{
    /* critical section - start */
    pthread_mutex_lock(&mutex);
    devready = device_state == DEVICE_STATE_READY ? 1 : 0 ;
    pthread_mutex_unlock(&mutex);
    /* critical section - end */
    rcprint("Device State Change Callback Invoked", (ULONG)device_state );
    appstatechange();
}

static void cleanup(const char *msg, ULONG ercode)
{
    syslog( LOG_USER, "%s: %s (0x%lX)", __func__, msg, ercode );

    if( sdkbound )
        executeApiProc(eQCWWANDisconnect);

    pthread_mutex_destroy(&mutex);
    pthread_cond_destroy(&cond);

    free(sdkbinpath);

    if( eQCWWAN_ERR_NONE != ercode)
        exit(EXIT_FAILURE);
    else
        exit(EXIT_SUCCESS);
}

/*------------------------------
  Network Access Service START
  ------------------------------*/
ULONG wInitiateNetworkRegistration(networkreg_t *pin)
{
  pin->regType = 1; /* automatic registration */
  pin->rat = 5; /* UMTS */
  rc = InitiateNetworkRegistration( pin->regType,
                                    pin->mcc,
                                    pin->mnc,
                                    pin->rat  );

  rcprint( __func__, rc );

  fprintf(  stderr, "regtype: %lu, mnc: %d, mnc: %d, rat: %lu\n",
            pin->regType,
            pin->mcc,
            pin->mnc,
            pin->rat );

  return rc;
}

ULONG wPerformNetworkScan(BYTE* pinstanceSize, networkscan_t* pinstances)
{
    rc = PerformNetworkScan(pinstanceSize, (BYTE *)pinstances);

    if( eQCWWAN_ERR_NONE == rc )
    {
        syslog(LOG_USER, "%s: pinstanceSize = %u\n", __func__, *pinstanceSize);
        for( int i =  *pinstanceSize ; i ; i-- )
        {
            fprintf( stderr,
                    "mcc: %d, mnc: %d, inuse: %lu, roaming: %lu forbidden:"\
                     "%lu, preferred: %lu, description: %s\n",
                     pinstances[i-1].mcc,
                     pinstances[i-1].mnc,
                     pinstances[i-1].inuse,
                     pinstances[i-1].roaming,
                     pinstances[i-1].forbidden,
                     pinstances[i-1].preferred,
                     pinstances[i-1].description );
        }
        nwreg.mcc = pinstances[0].mcc;
        nwreg.mnc = pinstances[0].mnc;
    }
    else
    {
        rcprint( __func__, rc );
    }

    return rc;
}
/*------------------------------
  Network Access Service END
  ------------------------------*/

/*-----------------------------
  Wireless Data Service START
  -----------------------------*/
ULONG wStopDataSession(ULONG sessionId)
{
  rc = StopDataSession(sessionId);
  rcprint(__func__, rc );
  return rc;
}

ULONG wStartDataSession(session_state_t *pss)
{
  ULONG technology = 1;
  rc = StartDataSession2( &technology, /* UMTS */
                          NULL, /* Primary DNS */
                          NULL, /* Secondary DNS */
                          NULL, /* Primary NBNS */
                          NULL, /* Secondary NBNS */
                          NULL, /* APN */
                          NULL, /* IP address */
                          NULL, /* authentication */
                          NULL, /* username */
                          NULL, /* password */
                          &pss->sessionid,
                          &pss->failurereason );

  rcprint(__func__, rc );
  fprintf( stderr, "session id: %lu, failurereason: %lu\n",
           pss->sessionid,
           pss->failurereason );

  return rc;
}

ULONG wGetSessionState(void *dummy)
{
    ULONG State;

    UNUSEDPARAM ( dummy );
    rc = GetSessionState( &State );
    rcprint(__func__, rc );
    if( eQCWWAN_ERR_NONE == rc )
        fprintf( stderr,"session state = %lu\n", State);

  return State;
}

#if 0
ULONG wSetDefaultProfile(profile_info_t *pprof)
{
    ULONG rc;
    rc =  SetDefaultProfile(   pprof->profileType,
                               pprof->pPDPType,
                               pprof->pIPAddress,
                               pprof->pPrimaryDNS,
                               pprof->pSecondaryDNS,
                               pprof->pAuthentication,
                               pprof->pName,
                               pprof->pAPNName,
                               pprof->pUsername,
                               pprof->pPassword );
  if( eQCWWAN_ERR_NONE != rc )
  {
      rcprint(__func__, rc);
  }

  return rc;
}
#endif
/*-----------------------------
  Wireless Data Service  END
  -----------------------------*/

/*-----------------------------
  Device Connectivity START
  -----------------------------*/
ULONG wQCWWANGetConnectedDevice(device_info_t* pdev)
{
  rc = QCWWAN2kGetConnectedDeviceID(
          DEV_NODE_SZ,
          pdev->deviceNode,
          DEV_KEY_SZ,
          pdev->deviceKey  );

  rcprint( __func__, rc );

  if( eQCWWAN_ERR_NONE == rc )
  {
      fprintf( stderr, "deviceNode: %s\ndeviceKey: %s\n",
               pdev->deviceNode,
               pdev->deviceKey  );

  }
  return rc;
}

ULONG wSLQSStart(void *dummy)
{
    UNUSEDPARAM ( dummy );

    /* Set SDK executable path */
    if( eQCWWAN_ERR_NONE != (rc = SetSDKImagePath(sdkbinpath) ) )
    {
        cleanup( __func__, rc );
    }

    /* establish app<->sdk IPC */
    rc = SLQSStart();
    if( eQCWWAN_ERR_SWICM_SOCKET_IN_USE == rc )
    {
        rcprint("Another APP is using the SDK", rc);
        cleanup( __func__, rc );
    }
    else if( eQCWWAN_ERR_NONE != rc )
    {
        if( eQCWWAN_ERR_NONE != executeApiProc(eSLQSKillSDKProcess) )
        {
            rcprint( __func__, rc );
        }
        else
        {
            rcprint( __func__, rc );
            if( eQCWWAN_ERR_NONE != (rc = SLQSStart()) )
            {
                cleanup( __func__, rc );
            }
            else
                rcprint("SDK process restarted", rc );
        }
    }
    else
    {
        syslog(LOG_USER, "%s: APP<->SDK IPC init successful\n", __func__);
    }

    /* Register for device state change notification */
    rc = SetDeviceStateChangeCbk( &devstatecb );
    if( eQCWWAN_ERR_NONE != rc )
    {
        cleanup( "APP failed to register for Device State Change notification", rc );
    }
    else
    {
        syslog( LOG_USER,
                "%s: APP registered for Device State Change notification\n",
                __func__ );
    }

#ifdef AUTOCONNECT
    if( eQCWWAN_ERR_NONE != (rc  = executeApiProc(eQCWWANEnumerateDevices)) )
    {
        rcprint( __func__, rc );
    }
    else if( eQCWWAN_ERR_NONE != (rc  = executeApiProc(eQCWWANConnect)) )
    {
        rcprint( __func__, rc );
    }
    if( eQCWWAN_ERR_NONE != (rc  = executeApiProc(eStartDataSession)) )
    {
         rcprint( __func__, rc );
    }
#endif
    return rc;
}

/* Kill SDK process when all else fails. This can occur, for
   example, if the application was started with non root
   priveleges. Subsequent tries to start up any application
   will fail because the SDK requires root permission to access
   /dev/qcqmi device special files.
*/
ULONG wSLQSKillSDKProcess(void *dummy)
{
    UNUSEDPARAM ( dummy );

    if( eQCWWAN_ERR_NONE != (rc = SLQSKillSDKProcess() ) )
    {
        perror(NULL);
        cleanup("Could not kill SDK process", -1);
    }

    rcprint( __func__, rc );
    sdkbound = 0;
    return 0;
}

/* Disconnect from Gobi API */
ULONG wQCWWANDisconnect(void *dummy)
{
    UNUSEDPARAM ( dummy );

    rc = QCWWANDisconnect();
    if( eQCWWAN_ERR_NONE != rc )
    {
        rcprint("APP failed to disconnect from SDK", rc);
    }
    else
    {
        /* application no longer bound to SDK */
        sdkbound = 0;
        appstatechange();
    }

    return rc;
}

/* Bind to SDK */
ULONG wQCWWANConnect(device_info_t* pdev)
{
    rc = QCWWANConnect( pdev->deviceNode, pdev->deviceKey );

    if( eQCWWAN_ERR_NONE == rc )
    {
        rcprint( __func__, rc );
        sdkbound = 1;
        appstatechange();
    }
    else
    {
        rcprint( __func__, rc );
    }

    return rc;
}

/* Enumerate QMI device */
ULONG wQCWWANEnumerateDevices(device_info_t* pdevices)
{
    BYTE DevicesSize = 1;
    ULONG rc;

    memset( pdevices, 0, sizeof(devices) );
    rc = QCWWAN2kEnumerateDevices(  &DevicesSize,
                                    (BYTE *)(pdevices) );

    rcprint( __func__, rc);
    syslog( LOG_USER, "#devices: %d deviceNode: %s deviceKey: %s",
            DevicesSize,
            pdevices->deviceNode,
            pdevices->deviceKey  );


    if( eQCWWAN_ERR_NONE == rc )
    {
        /* critical section - start */
        pthread_mutex_lock(&mutex);
        devready = 1;
        pthread_mutex_unlock(&mutex);
        /* critical section - end */
        appstatechange();
    }

    return rc;
}

/*-----------------------------
  Device Connectivity END
  -----------------------------*/

static int executeApiProc(int i)
{
    int dummy;
    memset( &dfltProf, 0, sizeof(dfltProf) );

    switch(i){
    case eSLQSStart:
        rc = wSLQSStart((int *)dummy);
        break;
    case eQCWWAN2kEnumerateDevices:
        rc = apiProcTable[i].proc(pdev);
        break;
    case eQCWWANConnect:
        rc = apiProcTable[i].proc(pdev);
        break;
    case eQCWWANDisconnect:
        rc = apiProcTable[i].proc(&dummy);
        break;
    case eQCWWANGetConnectedDeviceID:
        rc = apiProcTable[i].proc(pdev);
        break;
    case eGetSessionState:
        rc = apiProcTable[i].proc(&dummy);
        break;
    case eStartDataSession:
        rc = apiProcTable[i].proc(&ss);
        break;
    case eStopDataSession:
        rc = apiProcTable[i].proc((void *)ss.sessionid);
        break;
    case ePerformNetworkScan:
        rc = apiProcTable[i].proc(&instanceSize, instances);
        break;
    case eInitiateNetworkRegistration:
        rc = apiProcTable[i].proc(&nwreg);
        break;
#if 0
    case eSetDefaultProfile:
      rc = apiProcTable[i].proc(&dfltProf);
      break;
#endif
    case eSLQSKillSDKProcess:
        rc = apiProcTable[i].proc((void *)dummy);
        break;
    default:
        syslog(LOG_USER, "unhandled request, i = %d\n", i);
        break;
    }

    return rc;
}

static void menudisplay()
{
    putchar('\n');
    for( int i =  eSLQSStart ; strlen(apiProcTable[i].str) ; i++)
        printf("%d: %s\n", i, apiProcTable[i].str);
}

static int prompt(char *buf, int size)
{
    printf( "Enter sequence of space delimited functions to"\
            " execute using the indices above:\n>");

    if( fgets(buf, size, stdin) == 0)
        return -1;

    return 0;
}

static void init()
{
    /* initialize synchronization variables */
    pthread_mutex_init(&mutex, NULL);
    pthread_cond_init(&cond, NULL);

    executeApiProc(eSLQSStart);
}

int main(int argc, char **argv)
{
    if( argc < 2 )
    {
        fprintf( stderr, "usage: %s <path to sdk binary>\n", argv[0] );
        exit( EXIT_SUCCESS );
    }

    if( NULL == (sdkbinpath = malloc(strlen(argv[1]) + 1)) )
    {
        perror(__func__);
        exit( EXIT_FAILURE );
    }

    strncpy( sdkbinpath, argv[1], strlen(argv[1]) + 1);

    init();

    /* menu driver */
    while(1)
    {
        menudisplay();
        char lnbuf[1024];
        char *args[ARGS_MAX];
        if( prompt(lnbuf, ARGS_MAX) == -1 )
        {
            cleanup("exiting...", 0);
        }

        int i = 0;
        char delims[] = " \n";

        args[0] = strtok(lnbuf, delims);
        while( (args[++i] = strtok(NULL, delims)) != NULL);

        i = 0;
        while( args[i] != NULL )
        {
            int j = atoi(args[i]);
            if( j == eExit )
            {
                cleanup("Good bye!", eQCWWAN_ERR_NONE);
            }

            if( j < eSLQSStart )
            {
                syslog(LOG_USER, "%d: invalid menu option\n", j);
            }
            else
            {
                executeApiProc(j);
                args[i] = NULL; /* reset argument for next iteration */
            }
            i++; /* proceed to next argument */
        }
    }
}
